from django.core.exceptions import ValidationError
from django.test import TestCase
from django.db import transaction

from netaddr.core import AddrFormatError

from .models import NetworkThingy


class MACAddressFieldTestCase(TestCase):
    def test_insert_valid_macaddress(self):
        mac_example = '00:11:22:33:44:aa'
        x = NetworkThingy(mac=mac_example)
        x.save()
        qm = NetworkThingy.objects
        self.assertEquals(x.mac, mac_example)
        self.assertEquals(qm.get(mac=mac_example).mac, mac_example)
        self.assertEquals(qm.all().count(), 1)

    def test_insert_invalid_macaddress(self):
        invalid_mac = 'XX'
        with transaction.atomic():
            x = NetworkThingy()
            with self.assertRaises(ValidationError):
                x.mac = invalid_mac
                x.save()
        self.assertEquals(NetworkThingy.objects.all().count(), 0)



"""
A model for testing
"""
from django.db import models
from macaddress.fields import MACAddressField

class NetworkThingy(models.Model):
    mac = MACAddressField()

    def __unicode__(self):
        return "%s" % self.mac



import django
from django.core.exceptions import ValidationError
from django.db import models

from netaddr import EUI, AddrFormatError

from .formfields import MACAddressField as MACAddressFormField

from . import default_dialect, format_mac, mac_linux

import warnings

class MACAddressField(models.Field):
    description = "A MAC address validated by netaddr.EUI"
    empty_strings_allowed = False
    dialect = None

    def __init__(self, *args, **kwargs):
        self.integer = kwargs.pop('integer', True)
        if not self.integer: # If storing MAC address as string, set max_length to default (17) or use supplied kwarg value.
            kwargs['max_length'] = kwargs.get('max_length', 17)
        super(MACAddressField, self).__init__(*args, **kwargs)


    def deconstruct(self):
        ''' Django 1.7 migrations require this method
            https://docs.djangoproject.com/en/dev/howto/custom-model-fields/#field-deconstruction
        '''
        name, path, args, kwargs = super(MACAddressField, self).deconstruct()
        kwargs['integer'] = self.integer
        return name, path, args, kwargs

    @classmethod
    def set_dialect(cls, new_dialect_clazz):
        ''' Setting dialect for EUI (MAC addresses) globally to this Field
        class.
        Class new_dialect_clazz should (finally) extend
        netaddr.strategy.eui48.mac_eui48.
        '''
        warnings.warn(
            "The set_dialect method has been deprecated, in favor of the default_dialect utility function and "
            " settings.MACADDRESS_DEFAULT_DIALECT. See macaddress.__init__.py source or the project README for "
            "more information.",
            DeprecationWarning,
        )
        cls.dialect = new_dialect_clazz

    def get_prep_value(self, value):
        if value is None:
            return None
        if not isinstance(value, EUI):
            value = self.to_python(value)
            if self.integer:
                return int(value)
            return str(value)
        value.dialect = default_dialect(self)
        if self.integer:
            return int(value)
        return str(value)

    def get_internal_type(self):
        if self.integer:
            return 'BigIntegerField'
        return 'CharField'

    # @todo: remove context parameter when facing out django < 2.0 support
    def from_db_value(self, value, expression, connection, context=None):
        return self.to_python(value)

    def to_python(self, value):
        if value is None:
            return value
        if isinstance(value, EUI):
            value.dialect = default_dialect(value)
            return value
        try:
            return EUI(value, version=48, dialect=default_dialect())
        except (TypeError, ValueError, AddrFormatError):
            raise ValidationError(
                "This value must be a valid MAC address.")

    def formfield(self, **kwargs):
        defaults = {'form_class': MACAddressFormField}
        defaults.update(kwargs)
        return super(MACAddressField, self).formfield(**defaults)

    def get_prep_lookup(self, lookup_type, value):
        # data is stored internally as integer so searching as string
        # yeild 0 result. for example: useful for search in admin.
        if lookup_type in ('exact', 'iexact', 'icontains', 'icontains'):
            try:
                return self.get_prep_value(value)
            except AddrFormatError:
                return None
        elif lookup_type in ('in'):
            try:
                macs = []
                for mac in value:
                    macs += [self.get_prep_value(mac)]
                return macs
            except AddrFormatError:
                return None
        else:
            raise TypeError('Lookup type %r not supported.' % lookup_type)

if django.VERSION < (1, 8):
    from django.utils.six import add_metaclass
    MACAddressField = add_metaclass(models.SubfieldBase)(MACAddressField)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^macaddress\.fields\.MACAddressField"])
except ImportError:
    pass



from django.conf import settings

from netaddr import mac_unix, mac_eui48

import importlib
import warnings

class mac_linux(mac_unix):
    """MAC format with zero-padded all upper-case hex and colon separated"""
    word_fmt = '%.2X'

def default_dialect(eui_obj=None):
    # Check to see if a default dialect class has been specified in settings,
    # using 'module.dialect_cls' string and use importlib and getattr to retrieve dialect class. 'module' is the module and
    # 'dialect_cls' is the class name of the custom dialect. The dialect must either be defined or imported by the module's
    # __init__.py if the module is a package.
    from .fields import MACAddressField # Remove import at v1.4
    if hasattr(settings, 'MACADDRESS_DEFAULT_DIALECT') and not MACAddressField.dialect:
        module, dialect_cls = settings.MACADDRESS_DEFAULT_DIALECT.split('.')
        dialect = getattr(importlib.import_module(module), dialect_cls, mac_linux)
        return dialect
    else:
        if MACAddressField.dialect: # Remove this "if" statement at v1.4
            warnings.warn(
                "The set_dialect class method on MACAddressField has been deprecated, in favor of the default_dialect "
                "utility function and settings.MACADDRESS_DEFAULT_DIALECT. See macaddress.__init__.py source or the "
                "project README for more information.",
                DeprecationWarning,
            )
            return MACAddressField.dialect
        if eui_obj:
            return eui_obj.dialect
        else:
            return mac_linux

def format_mac(eui_obj, dialect):
    # Format a EUI instance as a string using the supplied dialect class, allowing custom string classes by
    # passing directly or as a string, a la 'module.dialect_cls', where 'module' is the module and 'dialect_cls'
    # is the class name of the custom dialect. The dialect must either be defined or imported by the module's __init__.py if
    # the module is a package.
    if not isinstance(dialect, mac_eui48):
        if isinstance(dialect, str):
            module, dialect_cls = dialect.split('.')
            dialect = getattr(importlib.import_module(module), dialect_cls)
    eui_obj.dialect = dialect
    return str(eui_obj)


from pkg_resources import get_distribution, DistributionNotFound
import os.path

try:
    _dist = get_distribution('django-macaddress')
except DistributionNotFound:
    __version__ = 'Please install this project with setup.py'
else:
    __version__ = _dist.version
VERSION = __version__   # synonym
